@@ -21,7 +21,7 @@
 *
 *****/
 
-#include "libmissing.h"
+#include "config.h"
 
 #include <stdio.h>
 #include <stdlib.h>
